/**
 * This file demonstrates what you need to include in your ray tracer to
 * utilize the parallel client/server applications.
 */

#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <stdlib.h>



void render(int x, int y, int *pixel);


//
// Your ray tracer will be sent arguments something like the following from
// the command line.
//
// ray_tracer.exe -parallel -scene yourscene.dat -path c:\rt_app\
//
main(int argc, char **argv)
{
	
	bool isParallel = false;
	char *path;
	char *sceneFile;
	char line[255];


	//
	// Parse the arguments
	//
	for (int count = 0; count < argc; count ++)
	{
		//
		// You can use -parallel to determine when to use
		// your regular ray tracer, or the parallel version.
		//
		// In the parallel version, you do not want to printf regular messages
		// to the standard output.  Instead, you should output those to
		// the standard error or a text file...
		//
		if (!strcmp(argv[count], "-parallel"))
			isParallel = true;
		
		//
		// This is the path to the SCENE file, not necessarily the ray tracer exe.
		// Therefore, you can have a scene and its textures stored within a sub folder.
		// As well, in your scene file, if you load external files like textures, you
		// should path them relative to where your scene file is located.
		//
		if (!strcmp(argv[count], "-path"))
		{
			path = argv[count + 1];
		}
		
		
		//
		// This is the scene file to be rendered.  You should use this
		// in conjunction with the path specified above.
		//
		if (!strcmp(argv[count], "-scene"))
		{
			sceneFile = argv[count + 1];
		}

	} // Parse the arguments




	if (!isParallel)
	{
		//
		// Execute your ray tracer normally without the parallel client/server
		//
		// ...
		//
		printf("Regular execution...\n");
		printf("\nPress any key to continue...");
		getch();
	}
	else
	{
		//
		// Load your scene file, getting the width and height of the image to be rendered
		//

		//loadScene( strcat(path, sceneFile) );

		int resX = 400;
		int resY = 300;


		int offset = 0;
		int length = 0;
		int x, y;


		while (true) // Keep taking commands from client app
		{
			//
			// Read in the offset and length of this segment to do
			//
			gets(line);
			
			if (strlen(line) == 0) break;


			if (!strcmp(line, "QUIT")) break; // Rendering complete



			//
			// We need to send the client the image resolution
			//
			if (!strcmp(line, "GETRES"))
			{
				printf("%d\n", resX);
				printf("%d\n", resY);
				fflush(stdout);	// It is important to flush the output
				
				continue;

			} // Server needs image resolution



			//
			// Otherwise, requesting an image segment
			//
			offset = atoi(line);
			
			gets(line);
			length = atoi(line);
			
			
			int pixel[3];

			
			//
			// Go through the length of the segment
			//
			for (int count = offset; count < offset + length; count ++)
			{
				//
				// Convert the offset to an x,y position
				//
				x = (int) (count % resX);
				y = (int) (count / resX);
				

				//
				// Shoot a ray through this pixel coordinate and return the colour
				//
				render(x, y, pixel);


				//
				// Send the pixel data back to the client
				//
				// Note: The RGB values must range from 0..255
				//
				printf("%d %d %d\n", pixel[0], pixel[1], pixel[2]);
				fflush(stdout);

			} // go through pixels in the current segment
			
			
			
		} // Keep looping until the client app decides to stop
		


	} // Parallel version of your ray tracer

	


	return 0;

} // main




void render(int x, int y, int *pixel)
{
	//
	// Send back a red pixel for now
	//
	pixel[0] = 255;
	pixel[1] = 0;
	pixel[2] = 0;

} // render